@@ -106,6 +106,7 @@ class Testbed:
         enable_apt_fallback: bool = True,
         shell_fail: bool = False,
         needs_internet: str = 'run',
+        insecure: bool = False,
     ) -> None:
         self.sp = None
         self.lastsend = None
@@ -130,6 +131,7 @@ class Testbed:
         self.apt_upgrade = apt_upgrade
         self.copy_files = copy_files
         self.initial_kernel_version = None
+        self.insecure = insecure
         # tests might install a different kernel; [(testname, reboot_marker, kver)]
         self.test_kernel_versions: List[Tuple[str, str, str]] = []
         # used for tracking kernel version changes
@@ -354,6 +356,23 @@ class Testbed:
             ) as reader:
                 self.check_exec(['sh', '-euc', reader.read(), 'sh', self.user])
 
+        if self.su_user is not None and self.su_user != 'root' and not any(c in ['isolation-machine', 'isolation-container'] for c in self.caps):
+            # scratch directory is owned by root and not world-writeable.
+            # (See the rules in README.virtualisation-server.)
+            #
+            # But, we like to run things (particularly, tests) as a normal user.
+            # That normal user needs to be able to write to this directory.
+            #
+            # Once upon a time, this was possible because the scratch directory was 1777.
+            # Nowadays, we make it possible by making it owned by the normal user.
+            #
+            # This means that we make root on the testbed completely trust self.user.
+            # This is OK because either:
+            #   self.user came from the testbed capability suggested-normal-user=
+            #   self.user came from the --user command line option
+            # and in both cases this trust relationship is documented there.
+            self.check_exec(['chown', self.su_user, '--', self.scratch])
+
         # determine testbed architecture
         self.dpkg_arch = self.check_exec(['dpkg', '--print-architecture'], True).strip()
         adtlog.info('testbed dpkg architecture: ' + self.dpkg_arch)
@@ -584,15 +603,6 @@ class Testbed:
     ) -> None:
         """
         Install test dependencies into testbed.
-
-        Dependencies on binary packages coming from the package under test are turned into
-        versioned dependencies. E.g., if src:foo is the package under test, and src:foo builds
-        bin:bar, then a test dependency on `bar` (possibly coming from @ expansion) is turned
-        into a dependency on `bar (= VERSION)`, where VERSION is the candidate version according
-        to the configured pinning. This is needed to: (1) ensure that the right version is
-        installed (`apt-get satisfy bar` does not upgrade bar if the dependency is already
-        satisfied), and (2) to always be strict on the package under test binaries, even if the
-        fallback mechanism or --no-strict-pinning relax the configured pinning.
         """
 
         adtlog.debug('install_deps: deps_new=%s' % deps_new)
@@ -601,54 +611,6 @@ class Testbed:
         if not deps_new:
             return
 
-        if not package_under_test_deps:
-            package_under_test_deps = []
-
-        # This requires the per-version pin features implemented in apt 1.1~exp9.
-        if self.apt_version >= Version("1.1~exp9"):
-            # Set of package names found in package_under_test_deps
-            my_packages: Set[str] = set()
-            for altgr in package_under_test_deps:
-                for package in altgr:
-                    my_packages.add(package)
-
-            apt_pins = ""
-            for package in my_packages:
-                candidate, priority = self.get_candidate(package)
-                if not candidate:
-                    continue
-
-                for altgr in deps_new:
-                    for i, dep in enumerate(altgr):
-                        altgr[i] = re.sub(fr"^({package})([\s(].*\))?(?=\s|$)", fr'\1 (= {candidate})', dep)
-
-                # Extra pinning is needed so that if the existing pinning gets modified by
-                # the fallback mechanism, the required version remain the highest-priority one.
-                # This is needed as APT only considers the highest-priority option when resolving
-                # dependencies, unless --no-strict-pinning is in use. We may be able to drop this
-                # pinning if we replace the apt fallback mechanism with --no-strict-pinning.
-                #
-                # Only pin if the candidate has the default 500 priority. If this is not the
-                # case, some other pin is already altering the priority, and we should not
-                # add another pin. This is important to not interfere with the publication
-                # of built binaries, which uses generic-form pins, see DebBinaries.publish().
-                if priority == 500:
-                    apt_pins += textwrap.dedent(
-                        f"""\
-                        Package: {package}
-                        Pin: version {candidate}
-                        Pin-Priority: 995
-
-                        """
-                    )
-
-            adtlog.debug(f"versioned install_deps: deps_new={deps_new}")
-
-            if apt_pins:
-                self._add_apt_preference(apt_pins, "autopkgtest-packages-under-test.pref")
-        else:
-            adtlog.warning("dependencies on packages under test left unversioned: testbed apt is too old")
-
         self.satisfy_dependencies_string(
             ', '.join([' | '.join(altgroup) for altgroup in deps_new]),
             'install-deps',
@@ -1387,6 +1349,10 @@ class Testbed:
             # This should come first so that it affects all CLI options
             argv.append('--debug')
 
+        if any(r in ['isolation-machine', 'isolation-container'] for r in test.restrictions):
+            # Must come early, because it influences behaviour of other options eg --artifacts
+            argv.append('--isolation')
+
         argv.extend([
             '--artifacts={}'.format(test_artifacts),
             '--chdir={}'.format(tree.tb),
@@ -2212,6 +2178,11 @@ class TestbedPath:
             if rc != 0:
                 # chowning doesn't work on all shared downtmps, try to chmod
                 # instead
+
+                # I believe this is only needed for autopkgtest-virt-unshare, which
+                # no longer advertises a downtmp at all.
+                # But we allow it still, with --insecure, in case someone needs it.
+                assert self.insecure
                 self.testbed.check_exec(['chmod', '-R', 'go+rwX', '--', self.tb])
 
     def copyup(self, check_existing=False):
